@@ -13,6 +13,7 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+
 """Accesses the google.cloud.translation.v3beta1 TranslationService API."""
 
 import functools
@@ -20,9 +21,11 @@ import pkg_resources
 import warnings
 
 from google.oauth2 import service_account
+import google.api_core.client_options
 import google.api_core.gapic_v1.client_info
 import google.api_core.gapic_v1.config
 import google.api_core.gapic_v1.method
+import google.api_core.gapic_v1.routing_header
 import google.api_core.grpc_helpers
 import google.api_core.operation
 import google.api_core.operations_v1
@@ -39,6 +42,7 @@ from google.cloud.translate_v3beta1.proto import translation_service_pb2
 from google.cloud.translate_v3beta1.proto import translation_service_pb2_grpc
 from google.longrunning import operations_pb2
 
+
 _GAPIC_LIBRARY_VERSION = pkg_resources.get_distribution(
     "google-cloud-translate"
 ).version
@@ -74,15 +78,6 @@ class TranslationServiceClient(object):
 
     from_service_account_json = from_service_account_file
 
-    @classmethod
-    def location_path(cls, project, location):
-        """Return a fully-qualified location string."""
-        return google.api_core.path_template.expand(
-            "projects/{project}/locations/{location}",
-            project=project,
-            location=location,
-        )
-
     @classmethod
     def glossary_path(cls, project, location, glossary):
         """Return a fully-qualified glossary string."""
@@ -93,6 +88,15 @@ class TranslationServiceClient(object):
             glossary=glossary,
         )
 
+    @classmethod
+    def location_path(cls, project, location):
+        """Return a fully-qualified location string."""
+        return google.api_core.path_template.expand(
+            "projects/{project}/locations/{location}",
+            project=project,
+            location=location,
+        )
+
     def __init__(
         self,
         transport=None,
@@ -100,6 +104,7 @@ class TranslationServiceClient(object):
         credentials=None,
         client_config=None,
         client_info=None,
+        client_options=None,
     ):
         """Constructor.
 
@@ -130,6 +135,9 @@ class TranslationServiceClient(object):
                 API requests. If ``None``, then default info will be used.
                 Generally, you only need to set this if you're developing
                 your own client library.
+            client_options (Union[dict, google.api_core.client_options.ClientOptions]):
+                Client options used to set user options on the client. API Endpoint
+                should be set through client_options.
         """
         # Raise deprecation warnings for things we want to go away.
         if client_config is not None:
@@ -148,6 +156,15 @@ class TranslationServiceClient(object):
                 stacklevel=2,
             )
 
+        api_endpoint = self.SERVICE_ADDRESS
+        if client_options:
+            if type(client_options) == dict:
+                client_options = google.api_core.client_options.from_dict(
+                    client_options
+                )
+            if client_options.api_endpoint:
+                api_endpoint = client_options.api_endpoint
+
         # Instantiate the transport.
         # The transport is responsible for handling serialization and
         # deserialization and actually sending data to the service.
@@ -156,6 +173,7 @@ class TranslationServiceClient(object):
                 self.transport = transport(
                     credentials=credentials,
                     default_class=translation_service_grpc_transport.TranslationServiceGrpcTransport,
+                    address=api_endpoint,
                 )
             else:
                 if credentials:
@@ -166,7 +184,7 @@ class TranslationServiceClient(object):
                 self.transport = transport
         else:
             self.transport = translation_service_grpc_transport.TranslationServiceGrpcTransport(
-                address=self.SERVICE_ADDRESS, channel=channel, credentials=credentials
+                address=api_endpoint, channel=channel, credentials=credentials
             )
 
         if client_info is None:
@@ -223,37 +241,44 @@ class TranslationServiceClient(object):
 
         Args:
             contents (list[str]): Required. The content of the input in string format.
-                We recommend the total contents to be less than 30k codepoints.
-                Please use BatchTranslateText for larger text.
+                We recommend the total content be less than 30k codepoints.
+                Use BatchTranslateText for larger text.
             target_language_code (str): Required. The BCP-47 language code to use for translation of the input
                 text, set to one of the language codes listed in Language Support.
             mime_type (str): Optional. The format of the source text, for example, "text/html",
-                 "text/plain". If left blank, the MIME type is assumed to be "text/html".
+                 "text/plain". If left blank, the MIME type defaults to "text/html".
             source_language_code (str): Optional. The BCP-47 language code of the input text if
                 known, for example, "en-US" or "sr-Latn". Supported language codes are
                 listed in Language Support. If the source language isn't specified, the API
                 attempts to identify the source language automatically and returns the
-                the source language within the response.
-            parent (str): Optional. Only used when making regionalized call.
-                Format:
-                projects/{project-id}/locations/{location-id}.
+                source language within the response.
+            parent (str): Required. Location to make a regional or global call.
 
-                Only custom model/glossary within the same location-id can be used.
-                Otherwise 400 is returned.
+                Format: ``projects/{project-id}/locations/{location-id}``.
+
+                For global calls, use ``projects/{project-id}/locations/global``.
+
+                Models and glossaries must be within the same region (have same
+                location-id), otherwise an INVALID\_ARGUMENT (400) error is returned.
             model (str): Optional. The ``model`` type requested for this translation.
 
-                The format depends on model type: 1. Custom models:
-                projects/{project-id}/locations/{location-id}/models/{model-id}. 2.
-                General (built-in) models:
-                projects/{project-id}/locations/{location-id}/models/general/nmt
-                projects/{project-id}/locations/{location-id}/models/general/base
+                The format depends on model type:
 
-                For global (non-regionalized) requests, use {location-id} 'global'. For
-                example, projects/{project-id}/locations/global/models/general/nmt
+                -  AutoML Translation models:
+                   ``projects/{project-id}/locations/{location-id}/models/{model-id}``
+
+                -  General (built-in) models:
+                   ``projects/{project-id}/locations/{location-id}/models/general/nmt``,
+                   ``projects/{project-id}/locations/{location-id}/models/general/base``
+
+                For global (non-regionalized) requests, use ``location-id`` ``global``.
+                For example,
+                ``projects/{project-id}/locations/global/models/general/nmt``.
 
                 If missing, the system decides which google base model to use.
-            glossary_config (Union[dict, ~google.cloud.translate_v3beta1.types.TranslateTextGlossaryConfig]): Optional. Glossary to be applied. The glossary needs to be in the same
-                region as the model, otherwise an INVALID\_ARGUMENT error is returned.
+            glossary_config (Union[dict, ~google.cloud.translate_v3beta1.types.TranslateTextGlossaryConfig]): Optional. Glossary to be applied. The glossary must be within the same
+                region (have the same location-id) as the model, otherwise an
+                INVALID\_ARGUMENT (400) error is returned.
 
                 If a dict is provided, it must be of the same form as the protobuf
                 message :class:`~google.cloud.translate_v3beta1.types.TranslateTextGlossaryConfig`
@@ -296,6 +321,19 @@ class TranslationServiceClient(object):
             model=model,
             glossary_config=glossary_config,
         )
+        if metadata is None:
+            metadata = []
+        metadata = list(metadata)
+        try:
+            routing_header = [("parent", parent)]
+        except AttributeError:
+            pass
+        else:
+            routing_metadata = google.api_core.gapic_v1.routing_header.to_grpc_metadata(
+                routing_header
+            )
+            metadata.append(routing_metadata)
+
         return self._inner_api_calls["translate_text"](
             request, retry=retry, timeout=timeout, metadata=metadata
         )
@@ -321,18 +359,26 @@ class TranslationServiceClient(object):
             >>> response = client.detect_language()
 
         Args:
-            parent (str): Optional. Only used when making regionalized call.
-                Format:
-                projects/{project-id}/locations/{location-id}.
+            parent (str): Required. Location to make a regional or global call.
 
-                Only custom model within the same location-id can be used.
-                Otherwise 400 is returned.
+                Format: ``projects/{project-id}/locations/{location-id}``.
+
+                For global calls, use ``projects/{project-id}/locations/global``.
+
+                Only models within the same region (has same location-id) can be used.
+                Otherwise an INVALID\_ARGUMENT (400) error is returned.
             model (str): Optional. The language detection model to be used.
-                projects/{project-id}/locations/{location-id}/models/language-detection/{model-id}
-                If not specified, default will be used.
+
+                Format:
+                ``projects/{project-id}/locations/{location-id}/models/language-detection/{model-id}``
+
+                Only one language detection model is currently supported:
+                ``projects/{project-id}/locations/{location-id}/models/language-detection/default``.
+
+                If not specified, the default model is used.
             content (str): The content of the input stored as a string.
             mime_type (str): Optional. The format of the source text, for example, "text/html",
-                "text/plain". If left blank, the MIME type is assumed to be "text/html".
+                "text/plain". If left blank, the MIME type defaults to "text/html".
             retry (Optional[google.api_core.retry.Retry]):  A retry object used
                 to retry requests. If ``None`` is specified, requests will not
                 be retried.
@@ -370,6 +416,19 @@ class TranslationServiceClient(object):
         request = translation_service_pb2.DetectLanguageRequest(
             parent=parent, model=model, content=content, mime_type=mime_type
         )
+        if metadata is None:
+            metadata = []
+        metadata = list(metadata)
+        try:
+            routing_header = [("parent", parent)]
+        except AttributeError:
+            pass
+        else:
+            routing_metadata = google.api_core.gapic_v1.routing_header.to_grpc_metadata(
+                routing_header
+            )
+            metadata.append(routing_metadata)
+
         return self._inner_api_calls["detect_language"](
             request, retry=retry, timeout=timeout, metadata=metadata
         )
@@ -394,24 +453,30 @@ class TranslationServiceClient(object):
             >>> response = client.get_supported_languages()
 
         Args:
-            parent (str): Optional. Used for making regionalized calls.
-                Format: projects/{project-id}/locations/{location-id}.
-                For global calls, use projects/{project-id}/locations/global.
-                If missing, the call is treated as a global call.
+            parent (str): Required. Location to make a regional or global call.
 
-                Only custom model within the same location-id can be used.
-                Otherwise 400 is returned.
+                Format: ``projects/{project-id}/locations/{location-id}``.
+
+                For global calls, use ``projects/{project-id}/locations/global``.
+
+                Only models within the same region (have same location-id) can be used,
+                otherwise an INVALID\_ARGUMENT (400) error is returned.
             display_language_code (str): Optional. The language to use to return localized, human readable names
-                of supported languages. If missing, default language is ENGLISH.
+                of supported languages. If missing, then display names are not returned
+                in a response.
             model (str): Optional. Get supported languages of this model.
+
                 The format depends on model type:
-                1. Custom models:
-                projects/{project-id}/locations/{location-id}/models/{model-id}.
-                2. General (built-in) models:
-                projects/{project-id}/locations/{location-id}/models/general/nmt
-                projects/{project-id}/locations/{location-id}/models/general/base
-                Returns languages supported by the specified model.
-                If missing, we get supported languages of Google general NMT model.
+
+                -  AutoML Translation models:
+                   ``projects/{project-id}/locations/{location-id}/models/{model-id}``
+
+                -  General (built-in) models:
+                   ``projects/{project-id}/locations/{location-id}/models/general/nmt``,
+                   ``projects/{project-id}/locations/{location-id}/models/general/base``
+
+                Returns languages supported by the specified model. If missing, we get
+                supported languages of Google general base (PBMT) model.
             retry (Optional[google.api_core.retry.Retry]):  A retry object used
                 to retry requests. If ``None`` is specified, requests will not
                 be retried.
@@ -445,6 +510,19 @@ class TranslationServiceClient(object):
         request = translation_service_pb2.GetSupportedLanguagesRequest(
             parent=parent, display_language_code=display_language_code, model=model
         )
+        if metadata is None:
+            metadata = []
+        metadata = list(metadata)
+        try:
+            routing_header = [("parent", parent)]
+        except AttributeError:
+            pass
+        else:
+            routing_metadata = google.api_core.gapic_v1.routing_header.to_grpc_metadata(
+                routing_header
+            )
+            metadata.append(routing_metadata)
+
         return self._inner_api_calls["get_supported_languages"](
             request, retry=retry, timeout=timeout, metadata=metadata
         )
@@ -515,25 +593,30 @@ class TranslationServiceClient(object):
 
                 If a dict is provided, it must be of the same form as the protobuf
                 message :class:`~google.cloud.translate_v3beta1.types.OutputConfig`
-            parent (str): Optional. Only used when making regionalized call.
-                Format:
-                projects/{project-id}/locations/{location-id}.
+            parent (str): Required. Location to make a regional call.
 
-                Only custom models/glossaries within the same location-id can be used.
-                Otherwise 400 is returned.
-            models (dict[str -> str]): Optional. The models to use for translation. Map's key is target language
-                code. Map's value is model name. Value can be a built-in general model,
-                or a custom model built by AutoML.
+                Format: ``projects/{project-id}/locations/{location-id}``.
+
+                The ``global`` location is not supported for batch translation.
+
+                Only AutoML Translation models or glossaries within the same region
+                (have the same location-id) can be used, otherwise an INVALID\_ARGUMENT
+                (400) error is returned.
+            models (dict[str -> str]): Optional. The models to use for translation. Map's key is target
+                language code. Map's value is model name. Value can be a built-in
+                general model, or an AutoML Translation model.
 
                 The value format depends on model type:
-                1. Custom models:
-                projects/{project-id}/locations/{location-id}/models/{model-id}.
-                2. General (built-in) models:
-                projects/{project-id}/locations/{location-id}/models/general/nmt
-                projects/{project-id}/locations/{location-id}/models/general/base
 
-                If the map is empty or a specific model is
-                not requested for a language pair, then default google model is used.
+                -  AutoML Translation models:
+                   ``projects/{project-id}/locations/{location-id}/models/{model-id}``
+
+                -  General (built-in) models:
+                   ``projects/{project-id}/locations/{location-id}/models/general/nmt``,
+                   ``projects/{project-id}/locations/{location-id}/models/general/base``
+
+                If the map is empty or a specific model is not requested for a language
+                pair, then default google model (nmt) is used.
             glossaries (dict[str -> Union[dict, ~google.cloud.translate_v3beta1.types.TranslateTextGlossaryConfig]]): Optional. Glossaries to be applied for translation.
                 It's keyed by target language code.
 
@@ -578,6 +661,19 @@ class TranslationServiceClient(object):
             models=models,
             glossaries=glossaries,
         )
+        if metadata is None:
+            metadata = []
+        metadata = list(metadata)
+        try:
+            routing_header = [("parent", parent)]
+        except AttributeError:
+            pass
+        else:
+            routing_metadata = google.api_core.gapic_v1.routing_header.to_grpc_metadata(
+                routing_header
+            )
+            metadata.append(routing_metadata)
+
         operation = self._inner_api_calls["batch_translate_text"](
             request, retry=retry, timeout=timeout, metadata=metadata
         )
@@ -660,6 +756,19 @@ class TranslationServiceClient(object):
         request = translation_service_pb2.CreateGlossaryRequest(
             parent=parent, glossary=glossary
         )
+        if metadata is None:
+            metadata = []
+        metadata = list(metadata)
+        try:
+            routing_header = [("parent", parent)]
+        except AttributeError:
+            pass
+        else:
+            routing_metadata = google.api_core.gapic_v1.routing_header.to_grpc_metadata(
+                routing_header
+            )
+            metadata.append(routing_metadata)
+
         operation = self._inner_api_calls["create_glossary"](
             request, retry=retry, timeout=timeout, metadata=metadata
         )
@@ -709,9 +818,9 @@ class TranslationServiceClient(object):
                 resource, this parameter does not affect the return value. If page
                 streaming is performed per-page, this determines the maximum number
                 of resources in a page.
-            filter_ (str): Optional. Filter specifying constraints of a list operation. For
-                example, ``tags.glossary_name="products*"``. If missing, no filtering is
-                performed.
+            filter_ (str): Optional. Filter specifying constraints of a list operation.
+                Filtering is not supported yet, and the parameter currently has no effect.
+                If missing, no filtering is performed.
             retry (Optional[google.api_core.retry.Retry]):  A retry object used
                 to retry requests. If ``None`` is specified, requests will not
                 be retried.
@@ -722,10 +831,10 @@ class TranslationServiceClient(object):
                 that is provided to the method.
 
         Returns:
-            A :class:`~google.gax.PageIterator` instance. By default, this
-            is an iterable of :class:`~google.cloud.translate_v3beta1.types.Glossary` instances.
-            This object can also be configured to iterate over the pages
-            of the response through the `options` parameter.
+            A :class:`~google.api_core.page_iterator.PageIterator` instance.
+            An iterable of :class:`~google.cloud.translate_v3beta1.types.Glossary` instances.
+            You can also iterate over the pages of the response
+            using its `pages` property.
 
         Raises:
             google.api_core.exceptions.GoogleAPICallError: If the request
@@ -748,6 +857,19 @@ class TranslationServiceClient(object):
         request = translation_service_pb2.ListGlossariesRequest(
             parent=parent, page_size=page_size, filter=filter_
         )
+        if metadata is None:
+            metadata = []
+        metadata = list(metadata)
+        try:
+            routing_header = [("parent", parent)]
+        except AttributeError:
+            pass
+        else:
+            routing_metadata = google.api_core.gapic_v1.routing_header.to_grpc_metadata(
+                routing_header
+            )
+            metadata.append(routing_metadata)
+
         iterator = google.api_core.page_iterator.GRPCIterator(
             client=None,
             method=functools.partial(
@@ -815,6 +937,19 @@ class TranslationServiceClient(object):
             )
 
         request = translation_service_pb2.GetGlossaryRequest(name=name)
+        if metadata is None:
+            metadata = []
+        metadata = list(metadata)
+        try:
+            routing_header = [("name", name)]
+        except AttributeError:
+            pass
+        else:
+            routing_metadata = google.api_core.gapic_v1.routing_header.to_grpc_metadata(
+                routing_header
+            )
+            metadata.append(routing_metadata)
+
         return self._inner_api_calls["get_glossary"](
             request, retry=retry, timeout=timeout, metadata=metadata
         )
@@ -881,6 +1016,19 @@ class TranslationServiceClient(object):
             )
 
         request = translation_service_pb2.DeleteGlossaryRequest(name=name)
+        if metadata is None:
+            metadata = []
+        metadata = list(metadata)
+        try:
+            routing_header = [("name", name)]
+        except AttributeError:
+            pass
+        else:
+            routing_metadata = google.api_core.gapic_v1.routing_header.to_grpc_metadata(
+                routing_header
+            )
+            metadata.append(routing_metadata)
+
         operation = self._inner_api_calls["delete_glossary"](
             request, retry=retry, timeout=timeout, metadata=metadata
         )
